
import java.sql.*;
import javax.sql.*;


public class GestioneSpedizioni {
    final String URL = "jdbc:mysql://localhost:3306";
    final String database = "Spedizioni";
    final String user = "root";
    final String password = "";
    final private Connection con;

    public GestioneSpedizioni() throws SQLException {
        con = DriverManager.getConnection(URL+"/"+database, user, password);
    }
    
    /* restituisce il codice della spedizione, 0 in caso di errore */
    public int nuovaSpedizione(String codice_mittente, String codice_destinatario, String codice_sede_partenza, String codice_sede_arrivo, double peso, double costo) {
        Statement stat;
        ResultSet result;
        int codice_spedizione;
        Timestamp data_ora_spedizione = new Timestamp(System.currentTimeMillis());
        Timestamp data_ora_consegna = new Timestamp(100, 0, 1, 0, 0, 0, 0); // 2000-01-01 00:00:00
        int isolation_level = Connection.TRANSACTION_NONE;

        try {
             //  inizio transazione con massimo livello di isolamento da eventuali transazioni concorrenti
             con.setAutoCommit(false);
             isolation_level = con.getTransactionIsolation();
             con.setTransactionIsolation(Connection.TRANSACTION_SERIALIZABLE);
             stat = con.createStatement();
             String query = "SELECT MAX(codice_spedizione) FROM Spedizioni;";
             result = stat.executeQuery(query);
             if (result.first())
                codice_spedizione = result.getInt(1) + 1;
             else
                 codice_spedizione = 1;
             result.close();
             stat.close();
        }
        catch (SQLException exception) {
            try {
                 // rollback transazione, ripristino livello isolamento e auto-commit
                 con.rollback();
                 con.setTransactionIsolation(isolation_level);
                 con.setAutoCommit(true);  
            }
            catch (SQLException e) {
            }
            return 0;
        }
        try {
            stat = con.createStatement();
            String command = "INSERT INTO Spedizioni(codice_spedizione, data_ora_spedizione, data_ora_consegna, peso, costo, mittente, destinatario, sede_partenza, sede_arrivo) VALUES (" + codice_spedizione + ", '" + data_ora_spedizione.toString() + "', '" + data_ora_consegna.toString() + "', " + peso + ", " + costo + ", '" + codice_mittente + "', '" + codice_destinatario + "', '" + codice_sede_partenza + "', '" + codice_sede_arrivo + "');";
            if (stat.executeUpdate(command) == 0) {
                // rollback transazione, ripristino livello isolamento e auto-commit
                con.rollback();
                con.setTransactionIsolation(isolation_level);
                con.setAutoCommit(true);
                return 0;
            }
            stat.close();
            // commit transazione, ripristino livello isolamento e auto-commit
            con.commit();
            con.setTransactionIsolation(isolation_level);
            con.setAutoCommit(true);
        }
        catch (SQLException exception) {
            try {
                 // rollback transazione, ripristino livello isolamento e auto-commit
                 con.rollback();
                 con.setTransactionIsolation(isolation_level);
                 con.setAutoCommit(true);  
            }
            catch (SQLException e) {
            }
            return 0;
        }
        return codice_spedizione;
    }

    /* restituisce true in caso di successo, false altrimenti */
    public boolean consegnaSpedizione(int codice_spedizione) {
        Statement stat;
        ResultSet result;
        Timestamp data_ora_consegna = new Timestamp(System.currentTimeMillis());

        try {
             stat = con.createStatement();
             String query = "SELECT * FROM Spedizioni WHERE codice_spedizione = " + codice_spedizione + " AND data_ora_consegna = '2000-01-01 00:00:00';";
             result = stat.executeQuery(query);
             if (result.first() == false) {
                 return false;
             }
             result.close();
             stat.close();
        }
        catch (SQLException exception) {
            return false;
        }
        try {
            stat = con.createStatement();
            String command = "UPDATE Spedizioni SET data_ora_consegna = '" + data_ora_consegna.toString() + "' WHERE codice_spedizione = " + codice_spedizione + ";";
            if (stat.executeUpdate(command) == 0) {
                return false;
            }
            stat.close();
        }
        catch (SQLException exception) {
            return false;
        }
        return true;
    }
    
    /* restituisce elenco spedizioni, null se nessuna spedizione soddisfa il criterio di selezione */
    public Spedizione[] spedizioniNonConsegnate() {
        Spedizione elenco_spedizioni[];
        Statement stat;
        ResultSet result;
        int numero_spedizioni;
        int conta_spedizioni;
        Cliente mittente;
        Cliente destinatario;
        Sede partenza;
        Sede arrivo;
        Spedizione spedizione;
        
        try { // conteggio spedizioni non consegnate
            stat = con.createStatement();
            String query = "SELECT COUNT(*) AS numero FROM Spedizioni WHERE data_ora_consegna = '2000-01-01 00:00:00';";
            result = stat.executeQuery(query);
            if (result.first()) {
                numero_spedizioni = result.getInt(1);
                if (numero_spedizioni > 0)
                    elenco_spedizioni = new Spedizione[numero_spedizioni];
                else
                    return null;
            }
            else
                return null;
            result.close();
            stat.close();
        }
        catch (SQLException exception) {
            return null;
        }
        try {
            stat = con.createStatement();
            String query = "SELECT * FROM Spedizioni, Clienti AS Mittenti, Clienti AS Destinatari, Sedi AS Partenze, Sedi AS Arrivi WHERE Spedizioni.mittente = Mittenti.codice_cliente AND Spedizioni.destinatario = Destinatari.codice_cliente AND Spedizioni.sede_partenza = Partenze.codice_sede AND Spedizioni.sede_arrivo = Arrivi.codice_sede AND data_ora_consegna = '2000-01-01 00:00:00';";
            result = stat.executeQuery(query);
            conta_spedizioni = 0;
            while (result.next()) {
                mittente = new Cliente();
                destinatario = new Cliente();
                partenza = new Sede();
                arrivo = new Sede();
                spedizione = new Spedizione();
                mittente.setCodice_cliente(result.getString("Mittenti.codice_cliente"));
                mittente.setRagione_sociale(result.getString("Mittenti.ragione_sociale"));
                mittente.setIndirizzo(result.getString("Mittenti.indirizzo"));
                mittente.setCAP(result.getString("Mittenti.CAP"));
                mittente.setCitta(result.getString("Mittenti.citta"));
                mittente.setTelefono(result.getString("Mittenti.telefono"));
                mittente.setEmail(result.getString("Mittenti.email"));
                
                destinatario.setCodice_cliente(result.getString("Destinatari.codice_cliente"));
                destinatario.setRagione_sociale(result.getString("Destinatari.ragione_sociale"));
                destinatario.setIndirizzo(result.getString("Destinatari.indirizzo"));
                destinatario.setCAP(result.getString("Destinatari.CAP"));
                destinatario.setCitta(result.getString("Destinatari.citta"));
                destinatario.setTelefono(result.getString("Destinatari.telefono"));
                destinatario.setEmail(result.getString("Destinatari.email"));
                
                partenza.setCodice_sede(result.getString("Partenze.codice_sede"));
                partenza.setIndirizzo(result.getString("Partenze.indirizzo"));
                partenza.setCAP(result.getString("Partenze.CAP"));
                partenza.setCitta(result.getString("Partenze.citta"));
                partenza.setTelefono(result.getString("Partenze.telefono"));
                partenza.setEmail(result.getString("Partenze.email"));
                
                arrivo.setCodice_sede(result.getString("Arrivi.codice_sede"));
                arrivo.setIndirizzo(result.getString("Arrivi.indirizzo"));
                arrivo.setCAP(result.getString("Arrivi.CAP"));
                arrivo.setCitta(result.getString("Arrivi.citta"));
                arrivo.setTelefono(result.getString("Arrivi.telefono"));
                arrivo.setEmail(result.getString("Arrivi.email"));
                
                spedizione.setMittente(mittente);
                spedizione.setDestinatario(destinatario);
                spedizione.setSede_partenza(partenza);
                spedizione.setSede_arrivo(arrivo);
                spedizione.setCodice_spedizione(result.getInt("codice_spedizione"));
                spedizione.setData_ora_spedizione(result.getTimestamp("data_ora_spedizione"));
                spedizione.setData_ora_consegna(result.getTimestamp("data_ora_consegna"));
                spedizione.setPeso(result.getDouble("peso"));
                spedizione.setCosto(result.getDouble("costo"));
                
                elenco_spedizioni[conta_spedizioni] = spedizione;
                conta_spedizioni++;
            }
            result.close();
            stat.close();
            return elenco_spedizioni;
        }
        catch (SQLException exception) {
            return null;
        }
    }
    
    /* restituisce elenco spedizioni, null se nessuna spedizione soddisfa il criterio di selezione */
    public Spedizione[] spedizioniOdierne(String codice_sede) {
        Spedizione elenco_spedizioni[];
        Statement stat;
        ResultSet result;
        int numero_spedizioni;
        int conta_spedizioni;
        Cliente mittente;
        Cliente destinatario;
        Sede partenza;
        Sede arrivo;
        Spedizione spedizione;
        
        try { // conteggio spedizioni spedite oggi dalla sede specifica
            stat = con.createStatement();
            String query = "SELECT COUNT(*) AS numero FROM Spedizioni WHERE YEAR(data_ora_spedizione) = YEAR(NOW()) AND MONTH(data_ora_spedizione) = MONTH(NOW()) AND DAY(data_ora_spedizione) = DAY(NOW()) AND sede_partenza = '" + codice_sede +"';";
            result = stat.executeQuery(query);
            if (result.first()) {
                numero_spedizioni = result.getInt(1);
                if (numero_spedizioni > 0)
                    elenco_spedizioni = new Spedizione[numero_spedizioni];
                else
                    return null;
            }
            else
                return null;
            result.close();
            stat.close();
        }
        catch (SQLException exception) {
            return null;
        }
        try {
            stat = con.createStatement();
            String query = "SELECT * FROM Spedizioni, Clienti AS Mittenti, Clienti AS Destinatari, Sedi AS Partenze, Sedi AS Arrivi WHERE Spedizioni.mittente = Mittenti.codice_cliente AND Spedizioni.destinatario = Destinatari.codice_cliente AND Spedizioni.sede_partenza = Partenze.codice_sede AND Spedizioni.sede_arrivo = Arrivi.codice_sede AND YEAR(data_ora_spedizione) = YEAR(NOW()) AND MONTH(data_ora_spedizione) = MONTH(NOW()) AND DAY(data_ora_spedizione) = DAY(NOW()) AND sede_partenza = '" + codice_sede +"';";
            result = stat.executeQuery(query);
            conta_spedizioni = 0;
            while (result.next()) {
                mittente = new Cliente();
                destinatario = new Cliente();
                partenza = new Sede();
                arrivo = new Sede();
                spedizione = new Spedizione();
                mittente.setCodice_cliente(result.getString("Mittenti.codice_cliente"));
                mittente.setRagione_sociale(result.getString("Mittenti.ragione_sociale"));
                mittente.setIndirizzo(result.getString("Mittenti.indirizzo"));
                mittente.setCAP(result.getString("Mittenti.CAP"));
                mittente.setCitta(result.getString("Mittenti.citta"));
                mittente.setTelefono(result.getString("Mittenti.telefono"));
                mittente.setEmail(result.getString("Mittenti.email"));
                
                destinatario.setCodice_cliente(result.getString("Destinatari.codice_cliente"));
                destinatario.setRagione_sociale(result.getString("Destinatari.ragione_sociale"));
                destinatario.setIndirizzo(result.getString("Destinatari.indirizzo"));
                destinatario.setCAP(result.getString("Destinatari.CAP"));
                destinatario.setCitta(result.getString("Destinatari.citta"));
                destinatario.setTelefono(result.getString("Destinatari.telefono"));
                destinatario.setEmail(result.getString("Destinatari.email"));
                
                partenza.setCodice_sede(result.getString("Partenze.codice_sede"));
                partenza.setIndirizzo(result.getString("Partenze.indirizzo"));
                partenza.setCAP(result.getString("Partenze.CAP"));
                partenza.setCitta(result.getString("Partenze.citta"));
                partenza.setTelefono(result.getString("Partenze.telefono"));
                partenza.setEmail(result.getString("Partenze.email"));
                
                arrivo.setCodice_sede(result.getString("Arrivi.codice_sede"));
                arrivo.setIndirizzo(result.getString("Arrivi.indirizzo"));
                arrivo.setCAP(result.getString("Arrivi.CAP"));
                arrivo.setCitta(result.getString("Arrivi.citta"));
                arrivo.setTelefono(result.getString("Arrivi.telefono"));
                arrivo.setEmail(result.getString("Arrivi.email"));
                
                spedizione.setMittente(mittente);
                spedizione.setDestinatario(destinatario);
                spedizione.setSede_partenza(partenza);
                spedizione.setSede_arrivo(arrivo);
                spedizione.setCodice_spedizione(result.getInt("codice_spedizione"));
                spedizione.setData_ora_spedizione(result.getTimestamp("data_ora_spedizione"));
                spedizione.setData_ora_consegna(result.getTimestamp("data_ora_consegna"));
                spedizione.setPeso(result.getDouble("peso"));
                spedizione.setCosto(result.getDouble("costo"));
                
                elenco_spedizioni[conta_spedizioni] = spedizione;
                conta_spedizioni++;
            }
            result.close();
            stat.close();
            return elenco_spedizioni;
        }
        catch (SQLException exception) {
            return null;
        }
    }
    
    /* restituisce spedizione, null se il codice non corrisponde a nessuna spedizione */
    public Spedizione statoSpedizione(int codice_spedizione) {
        Statement stat;
        ResultSet result;
        int numero_spedizioni;
        int conta_spedizioni;
        Cliente mittente;
        Cliente destinatario;
        Sede partenza;
        Sede arrivo;
        Spedizione spedizione;
        
        try {
            stat = con.createStatement();
            String query = "SELECT * FROM Spedizioni, Clienti AS Mittenti, Clienti AS Destinatari, Sedi AS Partenze, Sedi AS Arrivi WHERE Spedizioni.mittente = Mittenti.codice_cliente AND Spedizioni.destinatario = Destinatari.codice_cliente AND Spedizioni.sede_partenza = Partenze.codice_sede AND Spedizioni.sede_arrivo = Arrivi.codice_sede AND codice_spedizione = " + codice_spedizione +";";
            result = stat.executeQuery(query);
            if (result.first()) {
                mittente = new Cliente();
                destinatario = new Cliente();
                partenza = new Sede();
                arrivo = new Sede();
                spedizione = new Spedizione();
                mittente.setCodice_cliente(result.getString("Mittenti.codice_cliente"));
                mittente.setRagione_sociale(result.getString("Mittenti.ragione_sociale"));
                mittente.setIndirizzo(result.getString("Mittenti.indirizzo"));
                mittente.setCAP(result.getString("Mittenti.CAP"));
                mittente.setCitta(result.getString("Mittenti.citta"));
                mittente.setTelefono(result.getString("Mittenti.telefono"));
                mittente.setEmail(result.getString("Mittenti.email"));
                
                destinatario.setCodice_cliente(result.getString("Destinatari.codice_cliente"));
                destinatario.setRagione_sociale(result.getString("Destinatari.ragione_sociale"));
                destinatario.setIndirizzo(result.getString("Destinatari.indirizzo"));
                destinatario.setCAP(result.getString("Destinatari.CAP"));
                destinatario.setCitta(result.getString("Destinatari.citta"));
                destinatario.setTelefono(result.getString("Destinatari.telefono"));
                destinatario.setEmail(result.getString("Destinatari.email"));
                
                partenza.setCodice_sede(result.getString("Partenze.codice_sede"));
                partenza.setIndirizzo(result.getString("Partenze.indirizzo"));
                partenza.setCAP(result.getString("Partenze.CAP"));
                partenza.setCitta(result.getString("Partenze.citta"));
                partenza.setTelefono(result.getString("Partenze.telefono"));
                partenza.setEmail(result.getString("Partenze.email"));
                
                arrivo.setCodice_sede(result.getString("Arrivi.codice_sede"));
                arrivo.setIndirizzo(result.getString("Arrivi.indirizzo"));
                arrivo.setCAP(result.getString("Arrivi.CAP"));
                arrivo.setCitta(result.getString("Arrivi.citta"));
                arrivo.setTelefono(result.getString("Arrivi.telefono"));
                arrivo.setEmail(result.getString("Arrivi.email"));
                
                spedizione.setMittente(mittente);
                spedizione.setDestinatario(destinatario);
                spedizione.setSede_partenza(partenza);
                spedizione.setSede_arrivo(arrivo);
                spedizione.setCodice_spedizione(result.getInt("codice_spedizione"));
                spedizione.setData_ora_spedizione(result.getTimestamp("data_ora_spedizione"));
                spedizione.setData_ora_consegna(result.getTimestamp("data_ora_consegna"));
                spedizione.setPeso(result.getDouble("peso"));
                spedizione.setCosto(result.getDouble("costo"));
                
                result.close();
                stat.close();
                return spedizione;
            }
            else {
                result.close();
                stat.close();
                return null;
            }
        }
        catch (SQLException exception) {
            return null;
        }
    }
    
    
    
    public static void main(String args[]) {
        GestioneSpedizioni gestione_spedizioni;
        Spedizione elenco_spedizioni[];
        Spedizione spedizione;
        int codice;

        try {
            gestione_spedizioni = new GestioneSpedizioni();
        }
        catch (SQLException exception) {
            System.out.println("Errore connessione server DBMS.");
            return;
        }
        
        codice = gestione_spedizioni.nuovaSpedizione("00002", "00001", "00002", "00001", 0.200, 9.90);
        if (codice != 0) {
            System.out.println("Spedizione effettuata con codice " + codice + ".");
        }
        else {
            System.out.println("Errore spedizione.");
        }
        
        elenco_spedizioni = gestione_spedizioni.spedizioniNonConsegnate();
        if (elenco_spedizioni == null) {
            System.out.println("Nessuna spedizione non consegnata.");
        }
        else {
            System.out.println("Spedizioni non consegnate:");
            for (int i=0; i<elenco_spedizioni.length; i++) {
                System.out.println(elenco_spedizioni[i]);
            }
        }
        
        elenco_spedizioni = gestione_spedizioni.spedizioniOdierne("00002");
        if (elenco_spedizioni == null) {
            System.out.println("Nessuna spedizione spedita oggi.");
        }
        else {
            System.out.println("Spedizioni spedite oggi:");
            for (int i=0; i<elenco_spedizioni.length; i++) {
                System.out.println(elenco_spedizioni[i]);
            }
        }
        
        spedizione = gestione_spedizioni.statoSpedizione(codice);
        if (spedizione == null) {
            System.out.println("Spedizione inesistente.");
        }
        else {
            System.out.println("Spedizione:");
            System.out.println(spedizione);
        }
        
        if (gestione_spedizioni.consegnaSpedizione(codice)) {
            System.out.println("Spedizione consegnata.");
        }
        else {
            System.out.println("Errore consegna spedizione.");
        }
        
        spedizione = gestione_spedizioni.statoSpedizione(codice);
        if (spedizione == null) {
            System.out.println("Spedizione inesistente.");
        }
        else {
            System.out.println("Spedizione:");
            System.out.println(spedizione);
        }
    }
}
